/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
******************************************************************************
**                                                                            
**        Y     Y   OOOO    OOOO   DDDDD     AA
**         Y   Y   O    O  O    O  D    D   A  A
**          Y Y    O    O  O    O  D    D  A    A 
**           Y     O    O  O    O  D    D  AAAAAA
**           Y     O    O  O    O  D    D  A    A
**           Y      OOOO    OOOO   DDDDD   A    A
**
******************************************************************************
******************************************************************************/
/*****************************************************************************
*
*  MODULE :             Page Allocation management
*  SCOPE :              Implements allocation for YOODA volumes
*  FILE :               volallocator.cc
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma implementation
#endif

/* IMPORTS *******************************************************************/

#include <sys/types.h>

#ifdef __hpux__
extern "C" {
#endif
#include <sys/mman.h>
#ifdef __hpux__
}
#endif
  

#ifndef __rename_H
#include "rename.h"
#endif

#ifndef __volallocator_HXX
#include "volallocator.hxx"
#endif

#ifndef __config_H
#include "config.h"
#endif

#ifndef __yoofunc_HXX
#include "yoofunc.hxx"
#endif

#ifndef __tableofbits_HXX
#include "tableofbits.hxx"
#endif

/*****************************************************************************/

const int VolAllocator::SizeInPage = yoo_pageSize(PAGE_IN_VOL/BIT_IN_BYTE) ;


/*= EXTERNAL FUNCTION =======================================================*/
VolAllocator::VolAllocator(int fd)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  int size = SizeInPage * PAGE_SIZE ;
  this->_addr = yoo_mmap(0, size, PROT_READ|PROT_WRITE, MAP_SHARED, fd,0);
  this->_table = new TableOfBits(PAGE_IN_VOL, (char*)_addr) ;
}


/*= EXTERNAL FUNCTION =======================================================*/
VolAllocator::~VolAllocator()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  int tablesize = SizeInPage * PAGE_SIZE ;
  delete _table ;
  munmap(_addr, tablesize) ;
}


/*= EXTERNAL FUNCTION =======================================================*/
int VolAllocator::alloc(int number)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  int pos = this->_table->look(0, number) ;
  this->_table->set(pos, number) ;
  return pos + SizeInPage ;
}


/*= EXTERNAL FUNCTION =======================================================*/
void VolAllocator::freepage(int number)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  this->_table->clear(number-SizeInPage, 1) ;
}


/*= EXTERNAL FUNCTION =======================================================*/
int VolAllocator::isused(int pos, int len)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  return this->_table->isset(pos-SizeInPage, len) ;
}

